import json
import pandas as pd


def mask_column_with_function(pd_column, function):
    exec(function, globals())
    masked_column = pd_column.apply(maskInfo)
    return masked_column

def masking_all_column(encrypted_dataset,json_path):    
    with open(json_path, 'r') as json_file:
        init_result =  json.load(json_file)
    maskeddata = pd.read_csv(encrypted_dataset, low_memory=False)
    for col_info in init_result['content']:
            col_name = col_info['columnName']
            if col_info.get('sensitivity') == 1:
                # masked_df[col_name] = makeddata[col_name]
                print(f"Applying masking to sensitive column: {col_name}...")
                maskeddata[col_name] = mask_column_with_function(maskeddata[col_name], col_info['function'])            
            else:
                print(f"Skipping non-sensitive column: {col_name}")
                maskeddata[col_name]
    return maskeddata    




# masking_utils.py
import pandas as pd
from cryptography.fernet import Fernet
import os
import csv
import json

def encryption_dataset(csv_file_path,decryption_key_location,encrypted_dataset_location,json_path):
    with open(json_path, 'r') as json_file:
        init_result =  json.load(json_file)
    dataset = pd.read_csv(csv_file_path)
    sensitive_columns = []
    masking_functions = {}
    sensitive_column = None
    masking_function = None
    decryption_key = None


    for column_info in init_result["content"]:
        if column_info["sensitivity"] == 1:
            column_name = column_info["columnName"]
            masking_function = column_info["function"]
            sensitive_columns.append(column_name)
            masking_functions[column_name] = masking_function
   




    if sensitive_columns:
        decryption_keys = {}
        decryption_keys = {column: Fernet.generate_key() for column in sensitive_columns}        
        for column in sensitive_columns:
            cipher_suite = Fernet(decryption_keys[column])
            new_column_name = f"{column}_encrypted"
            dataset[new_column_name] = dataset[column].apply(lambda x: cipher_suite.encrypt(x.encode()).decode())
            
            duplicate_mask = dataset[column_name].notna()
            duplicated_rows = dataset[duplicate_mask]
            duplicated_df = pd.concat([dataset, duplicated_rows], ignore_index=True)
        result_df = pd.concat([dataset, duplicated_df], ignore_index=True)
        #input part 
        decryption_keys_dir = decryption_key_location
        ####
        for column, key in decryption_keys.items():
            key_file_path = os.path.join(decryption_keys_dir, f"{column}_decryption_key.txt")
            with open(key_file_path, "wb") as key_file:
                key_file.write(key)
        decryption_keys_csv_path = os.path.join(decryption_keys_dir, "decryption_keys.csv")
        with open(decryption_keys_csv_path, "w", newline='') as csvfile:
            fieldnames = ['Column', 'DecryptionKey']
            writer = csv.DictWriter(csvfile, fieldnames=fieldnames)

            writer.writeheader()
            for column, key in decryption_keys.items():
                writer.writerow({'Column': column, 'DecryptionKey': key.decode()})
    
    
    #output path
    masked_encrypted_csv_path = encrypted_dataset_location  
    result_df.to_csv(masked_encrypted_csv_path, index=False)
    return masked_encrypted_csv_path









import pandas as pd
from cryptography.fernet import Fernet
import json
import re

class DataEncryptorDecryptor:
    def __init__(self, decryptionkey,data,generated_json):
        self.decryptionkey = decryptionkey
        self.data = data
        self.generated_json = generated_json

    def encrypt_data(self):
        data1 = self.data
        if  isinstance(data1, pd.DataFrame):
            data = data1
        else:
            data = data1.toPandas()
        key = self.decryptionkey
        cipher_suite = Fernet(key)
        init_result = self.generated_json
        sensitive_columns = []
        masking_functions = {}

        for column_info in init_result["content"]:
            if column_info["sensitivity"] == 1:
                column_name = column_info["columnName"]
                masking_function = column_info["function"]
                sensitive_columns.append(column_name)
                masking_functions[column_name] = masking_function

        if sensitive_columns:
            for column in sensitive_columns:
                # new_column_name = f"{column}_encrypted"
                data[column_name] =  data[column].apply(lambda x: cipher_suite.encrypt(str(x).encode()).decode() if isinstance(x, (str, int, float)) else x)
                # duplicate_mask = data[column].notna()
                # duplicated_rows = data[duplicate_mask].copy()
                # data[column_name] = data[column].apply(
                #     lambda x: cipher_suite.encrypt(x.encode()).decode()
                # )
                result_df = data
            return result_df
# This function is designed for Pandas DataFrames; it requires the data to be in that format for processing
    def decrypt_data(self, sensitive_columns, masked_encrypted_csv_path):
        dataset = pd.read_csv(masked_encrypted_csv_path, low_memory=False)

        decryption_key = self.decryptionkey
        if decryption_key:
            cipher_suite = Fernet(decryption_key)
            for sensitive_column in sensitive_columns:
                dataset[sensitive_column] = dataset[sensitive_column].apply(
                    lambda x: cipher_suite.decrypt(x.encode()).decode()
                )
        else:
            print("Decryption key not valid")

        return dataset
# masks the data
    def masking_all_column(self, encrypted_dataset):
        init_result= self.generated_json
        def mask_column_with_function(pd_column, function):
            exec(function, globals())
            masked_column = pd_column.apply(maskInfo)
            return masked_column

        # init_result = self.generated_json
        maskeddata = encrypted_dataset.copy()

        for col_info in init_result['content']:
            col_name = col_info['columnName']
            if col_info.get('sensitivity') == 1:
                print(f"Applying masking to sensitive column: {col_name}...")
                maskeddata[col_name] = mask_column_with_function(maskeddata[col_name], col_info['function'])
            else:
                print(f"Skipping non-sensitive column: {col_name}")

        return maskeddata
# This handles both encryption and masking logic together 
    def encryption_masking(self):
        data1 = self.data
        if  isinstance(data1, pd.DataFrame):
            data = data1
        else:
            data = data1.toPandas()

        def mask_column_with_function(pd_column, function):
            exec(function, globals())
            # masked_column = pd_column.apply(maskInfo)
            masked_column = pd_column.astype(str).apply(maskInfo)
            # masked_column = pd_column.apply(lambda x: maskInfo(x))
            return masked_column
        key = self.decryptionkey
        cipher_suite = Fernet(key)
        init_result = self.generated_json
        sensitive_columns = []
        masking_functions = {}

        for column_info in init_result["content"]:
            if column_info["sensitivity"] == 1:
                column_name = column_info["columnName"]
                masking_function = column_info["function"]
                sensitive_columns.append(column_name)
                masking_functions[column_name] = masking_function
            
        if sensitive_columns:
            key = self.decryptionkey
            for column in sensitive_columns:
                new_column_name = f"{column}_encrypted"
                cipher_suite = Fernet(key)
                data[new_column_name] = data[column].apply(lambda x: cipher_suite.encrypt(str(x).encode()).decode() if isinstance(x, (str, int, float)) else x)
                duplicate_mask = data[column_name].notna()  
                duplicated_rows = data[duplicate_mask]
                duplicated_df = pd.concat([data, duplicated_rows], ignore_index=True)
                
                result_df = pd.concat([data, duplicated_df], ignore_index=True)
                maskeddata = result_df
                for col_info in init_result['content']:
                    col_name = col_info['columnName']
                    
                    if col_info.get('sensitivity') == 1:          
                        maskeddata[col_name] = mask_column_with_function(maskeddata[col_name],col_info['function'])          
                    else:
                        pass
            # maskeddata = spark.createDataFrame(maskeddata)
            return maskeddata




from setuptools import setup, find_packages

setup(
    name='maskingPackage',
    version='1.0.10',
    packages=find_packages(),
    install_requires=[
        'pandas',  # Add any dependencies here
        'cryptography'
    ],
)
